#!/bin/sh
# update /usr/bin/ecc_core

PATH=/bin:/sbin:/usr/bin:/bin:/usr/sbin:/usr/local/bin
export PATH

REQUIRED_SP=SP1

lng=$1
case $lng in
   local) ;;
   de) ;;
   en) ;;
   *) lng="de" ;;
esac

update() {
   rm -f /tmp/REBOOT

   if [ ! -x /usr/bin/opkg -a -x /usr/bin/opkg-cl ]; then
      ln /usr/bin/opkg-cl /usr/bin/opkg
   fi

   if [ ! -x /usr/bin/opkg ]; then
      echo "================================================================="
      echo "===== es fehlt das Programm 'opkg' fr den Updateprozess! ======="
      echo "================================================================="

      case $lng in
	 de) lcdmsg -m "OPKG ist nicht\ninstalliert!" ;;
	 en) lcdmsg -m "OPKG is not\ninstalled!" ;;
      esac
      sleep 3

      exit 1
   fi

   # laufende Prozesse beenden
   if [ -f /var/run/eccweb.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/eccweb.pid
   fi
   if [ -f /var/run/ecc_core.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_core.pid
   fi
   if [ -f /var/run/ecc_usv.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_usv.pid
   fi
   if [ -f /var/run/ecc_modbus.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_modbus.pid
   fi
   if [ -f /var/run/ecc_bacnet.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_bacnet.pid
   fi
   if [ -f /var/run/ecc_knx.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_knx.pid
   fi

   for pkg in *.opk; do
      pkgname=${pkg#/mnt/}
      pkgname=${pkgname%.opk}
      version=${pkgname#*_}
      version=${version%_*}
      pkgname=${pkgname%%_*}
      instver=`/usr/bin/opkg status ${pkgname} | fgrep "Version:"`
      instver=${instver#Version: }
      if [ -z "${instver}" ]; then instver="0.00"; fi
      echo -n ${pkgname} ${version} installed ${instver} ....

      # compare-versions arbeitet irgendwie falsch herum. Die Abfrage
      # mit dem '<' ist in wirklichkeit TRUE ($? == 0), wenn ${version}
      # GRSSER als ${instver} ist !!!
      /usr/bin/opkg compare-versions ${version} "=" ${instver}
      if [ $? -eq 0 ]; then
         echo "is installing"
	 case $lng in
	    de) lcdmsg -m "installiere\n${pkgname} ${version}" ;;
	    en) lcdmsg -m "installing\n${pkgname} ${version}" ;;
	 esac
	 /usr/bin/opkg install --force-downgrade ${pkg}
      else
         echo "is up to date"
      fi
   done

   if [ -f version.sql ]; then
      sqlite3 /data/ecc/ecc.db < version.sql
   fi

   if [ -f ID.tgz -a -x /usr/bin/cfg_reader ]; then
      case $lng in
         de) lcdmsg -m "CONFIG-IDs einspielen" ;;
         en) lcdmsg -m "importing CONFIG-IDs" ;;
      esac
      mkdir ID
      cd ID
      tar -zxf ../ID.tgz
      /usr/bin/cfg_reader *.node
   fi

   case $lng in
      de) lcdmsg -m "... Speichern wird\nabgeschlossen" ;;
      en) lcdmsg -m "... finishing\nstorage operation" ;;
   esac
   /bin/sync

# Mit der folgenden Zeile knnte man Skriptgesteuert die Ethernet-MAC aus dem I2C-EEPROM lesen ...
# Es mte aber auch eine Kontrolle auf eine gltige MAC erfolgen, falls gar kein EEPROM eingeltet ist!
   if false; then
      mac=`fw_printenv -n ethaddr`
      if [ -z "${mac}" ]; then
	 mac=`od -t x1 -A n -j 250 -N 6 /sys/bus/i2c/devices/0-0050/eeprom | sed -e 's/^ //' -e 's/ /:/g'`
	 echo "Setze Ethernet-MAC auf ${mac}"
	 fw_setenv ethaddr ${mac}
      else
	 echo "MAC ist schon auf ${mac} gesetzt"
      fi
   fi

   if [ -f /tmp/REBOOT ]; then
      case $lng in
	 de) lcdmsg -m "fertig... Neustart\nnach Stick abziehen" ;;
	 en) lcdmsg -m "ready... restart\nafter removing Stick" ;;
      esac
   else
      case $lng in
	 de) lcdmsg -m "fertig...\nbitte Stick abziehen" ;;
	 en) lcdmsg -m "ready...\nplease remove Stick" ;;
      esac
   fi
}

rc=0
echo "Running Update Check"

if [ -x /usr/bin/lcdmsg ]; then

   if [ -n "${REQUIRED_SP}" -a ! -f "/etc/${REQUIRED_SP}" ]; then
      echo "ServicePack ${REQUIRED_SP} is missing!"
      if [ "$lng" = "local" ]; then
	 rc=-1
      else
	 killall lcd_manager
	 case $lng in
	    de) lcdmsg -m "Es wird das\nServicePack ${REQUIRED_SP}\nbentigt!" -c "Abbruch"
		rc=$?
		;;
	    en) lcdmsg -m "Required ServicePack\n${REQUIRED_SP} is missing!" -c "Cancel"
		rc=$?
		;;
	 esac
      fi
   else
      if [ "$lng" = "local" ]; then
	 rc=0
      else
	 killall lcd_manager
	 case $lng in
	    de) lcdmsg -m "Mchten Sie das\nUpdate durchfhren?" -c "Abbruch" -o "UPDATE"
		rc=$?
		;;
	    en) lcdmsg -m "Would you like to\napply the Update?" -c "Cancel" -o "UPDATE"
		rc=$?
		;;
	 esac
	 echo "Selection: $rc"
      fi
   fi

   case $rc in
      0) case $lng in
            de) lcdmsg -m "Update luft ..." ;;
            en) lcdmsg -m "Update running ..." ;;
	 esac
	 update
# Reparatur des vermurksten DB-Updates ecc_up28.sql
# /usr/bin/sqlite3 /data/ecc/ecc.db "delete from ecc_settings;"
	 ;;
      *) case $lng in
      	    de) lcdmsg -m "Abgebrochen ...\nbitte Stick abziehen" ;;
      	    en) lcdmsg -m "Aborted ...\nplease remove Stick" ;;
	 esac
	 rc=2
	 ;;
   esac
else

   echo "================================================================="
   echo "==== es fehlt das Programm 'lcdmsg' fr den Updateprozess! ======"
   echo "================================================================="

   exit 1

fi

chmod 755 /etc/init.d/S*

sync
echo "... FINISHED ..."

exit $rc
